// Flexradioconnect
// Copyright 2016,2017,2018,2019 Hardy Lau, DL1GLH
//
// v190219-0.01 Alpha
//
// Standalone interface for interconnection of
// + Flexradio 6300/6500/6700 
// + S.P.E. Expert 2K-FA
// + Ultrabeam dynamic antenna
//
// Running on Banana Pro
// This part does connect to EX4TX ARS-USB via TCPIP (cgi)
//

/*
 Copyright 2016,2017,2018,2019 Hardy Lau, DL1GLH

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its contributors
    may be used to endorse or promote products derived from this software without
    specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#define _GNU_SOURCE
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include<signal.h>
#include<unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define DEBUG
#undef DEBUG

// EA4TX ARS-USB with network accessable port (TCPIP)
// YAESU Commands
#define ROTORSERVER "localhost"
#define ROTORPORT 6000

//Main
//
int main( int argc, char *argv[])
{
   int sockfd, portno, n;
   FILE *sock;
   struct sockaddr_in serv_addr;
   struct hostent *server;
   fd_set rfds;
   struct timeval tv;
   int retval;
   int i;
   int neuerichtung = -1;
   char buffer[200];
   struct itimerval timer;

   /* Configure the timer to expire after 800 msec... the alarm will kill the process */
   timer.it_value.tv_sec = 0;
   timer.it_value.tv_usec = 800000;
   timer.it_interval.tv_sec = 0;
   timer.it_interval.tv_usec = 0;
   setitimer (ITIMER_REAL, &timer, NULL);

   printf( "Content-type: text/html\n\n");
   fflush(stdout);
   printf( "<html><head></head><body>\n");
   
   if( getenv("REQUEST_METHOD") != NULL && !strcmp( "GET", getenv("REQUEST_METHOD"))) {
     if( getenv("QUERY_STRING") != NULL) {
       strcpy( buffer, getenv("QUERY_STRING"));
       if( strstr( buffer, "turn=")) {
         sscanf( buffer, "turn=%d", &neuerichtung); 
       }
     }
   }
 
   /* Create a socket point */
   sockfd = socket(AF_INET, SOCK_STREAM, 0);
   
   if (sockfd < 0) {
      perror("ERROR opening socket");
      exit(1);
   }
	
   server = gethostbyname(ROTORSERVER);
   
   if (server == NULL) {
      fprintf(stderr,"No such host name - trying ip number...\n");
      server = gethostbyaddr((char *)inet_addr(ROTORSERVER), 4, AF_INET);
      if(server == NULL) {
        perror("ERROR - No such host...\n");
        exit(1);
      }
   }
   
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
   serv_addr.sin_port = htons(ROTORPORT);
   
   /* Now connect to the server with rotor control */
   if (connect(sockfd, (struct sockaddr*)&serv_addr, sizeof(serv_addr)) < 0) {
      perror("ERROR connecting");
      exit(1);
   }

   /* Read direction after Connect
   */
   
   if( write(sockfd, "C\r", strlen( "C\r")) <= 0) {
     perror("ERROR writing");
     exit(1);
   }

   n = 0;
   do {
     if( read( sockfd, buffer + n, 1) < 1) {
       printf( "Rotor OFFLINE");
       printf( "</body></html>");
       perror("ERROR reading");
       exit(1);
     }
     n++;
     buffer[n] = '\0';
   } while( n < 6);
   buffer[n-1] = '\0';
   printf( "Rotor direction: %s", buffer+2);

   /* Set direction if requested
   */
   if( neuerichtung >= 0) {
     sprintf( buffer, "M%03d\r", neuerichtung);
     write(sockfd, buffer, strlen( buffer));
   }
   if( neuerichtung == -2) {
     sprintf( buffer, "S\r");
     write(sockfd, buffer, strlen( buffer));
   }

   close( sockfd);

   printf( "</body></html>");
}
