// Flexconnect
// Copyright 2016 Hardy Lau, DL1GLH
//
// v171213-0.93 Alpha
//
// Standalone interface for interconnection of
// + Flexradio 6300/6500/6700
// + S.P.E. Expert 2K-FA
// + Ultrabeam dynamic antenna
//
// Running on Banana Pro
// This part (process) does connect to S.P.E Expert 2K-FA linear
//

/*
 Copyright 2016,2017 Hardy Lau, DL1GLH

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its contributors
    may be used to endorse or promote products derived from this software without
    specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#define _GNU_SOURCE
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#define DEBUG
#undef DEBUG

//read a file /run/flexconnect/frequency-%1.6lf
//
double getfrequency()
{
 DIR *dir;
 struct dirent *dp;
 char *file_name;
 char file_name_full_path[200];
 int fd;
 double frequency = 0.0;
 int x = 30;

 dir = opendir( "/run/flexconnect");
 if( dir < 0) return( 0.0); 
 
 while ((dp=readdir(dir)) != NULL && x--) {
   file_name = dp->d_name;
   if( strstr( file_name, "frequency-")) {
     sscanf( file_name, "frequency-%lf", &frequency);
   }
 }
 closedir( dir);
 return( frequency);
}

//Main
//
int main( int argc, char *argv[])
{
  int i;
  int tty;
  int baud = B19200;
  struct termios newtio;
  char buffer[255];
  double frequency, oldfrequency;
  
  if (argc < 3) {
     fprintf(stderr,"usage %s tty_device baud\n", argv[0]);
     exit(0);
  }
  
  sscanf( argv[2], "%d", &baud);
  tty = open( argv[1], O_RDWR | O_NOCTTY ); 
  if (tty <0) { 
    perror(argv[1]); 
    exit(-1); 
  }

  bzero(&newtio, sizeof(newtio)); /* clear struct for new port settings */
        
  newtio.c_cflag = B19200 | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR | ICRNL;
  newtio.c_oflag = 0;
  newtio.c_lflag = ICANON;
  tcflush(tty, TCIFLUSH);
  tcsetattr(tty,TCSANOW,&newtio); 

  while( 1) {
    frequency = getfrequency();
    sprintf( buffer, "FA%011.0lf;", frequency * 1000000.0);
    if( frequency != oldfrequency) {
#ifdef DEBUG
      printf( "%s\n", buffer);
#endif
      oldfrequency = frequency;
    }
    if( !write( tty, buffer, strlen(buffer))) exit(1);
    sleep(1);
  }
}
