// Flexconnect
// Copyright 2016,2017,2018,2019 Hardy Lau, DL1GLH
//
// v171213-0.93 Alpha
// v190529-0.94 Alpha
//
// Standalone interface for interconnection of
// + Flexradio Series 6x00 (V1 & V2 & V3)
// + S.P.E. Expert 2K-FA
// + Ultrabeam dynamic antenna
//
// Running on Banana Pro
// This part (process) does connect to Flexradio
//

/*
 Copyright 2016,2017,2018,2019 Hardy Lau, DL1GLH

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.

 3. Neither the name of the copyright holder nor the names of its contributors 
    may be used to endorse or promote products derived from this software without 
    specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
 OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#define  _GNU_SOURCE
#define __USE_GNU
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <dirent.h>
#include <signal.h>
#include <time.h>

#define DEBUG
#undef DEBUG

//Create a file /run/flexconnect/frequency-%1.6lf
//
int tellfrequency( double frequency)
{
 DIR *dir;
 struct dirent *dp;
 char *file_name;
 char file_name_full_path[200];
 char file_name_full_path_new[200];
 int fd;

 //Create dir - doesn't matter if already exist
 mkdir( "/run/flexconnect", 0777);
 
 //Create file with frequency information if frequency > 0.0
 if( frequency >= 0.0) {
   sprintf( file_name_full_path_new, "/run/flexconnect/frequency-%1.6lf", frequency);
   fd = creat( file_name_full_path_new, 0777);
   close( fd);
 } else {
   sprintf( file_name_full_path_new, "/run/flexconnect/frequency");
 }

 //Delete all old frequency files
 dir = opendir( "/run/flexconnect");
 while ((dp=readdir(dir)) != NULL) {
   file_name = dp->d_name;
   if( strstr( file_name, "frequency-")) {
     strcpy( file_name_full_path, "/run/flexconnect/");
     strcat( file_name_full_path, file_name);
     //Delete all other frequency files
     if( strcmp( file_name_full_path_new, file_name_full_path)) {
       unlink( file_name_full_path);
     }
   }
 }
 closedir( dir);
}

//Main
//
int main( int argc, char *argv[])
{
   int sockfd, portno, n;
   FILE *sock;
   struct sockaddr_in serv_addr;
   struct hostent *server;
   fd_set rfds;
   struct timeval tv;
   int retval;
   int slicetx = -1;
   int sliceactive = -1;
   double slice_freq[8];
   int slice_active[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
   int slice_tx[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
   int i, j;
   char handle[20], version[20];
   unsigned int sequence = 3;
   time_t lastping;
  
   char buffer[2048], line[2048], *x, comp[50];
  
   tellfrequency( -1.0);
 
   if (argc < 3) {
      fprintf(stderr,"usage %s hostname port\n", argv[0]);
      exit(0);
   }
	
   portno = atoi(argv[2]);
   
   /* Create a socket point */
   sockfd = socket(AF_INET, SOCK_STREAM, 0);
   
   if (sockfd < 0) {
      perror("ERROR opening socket");
      exit(1);
   }
	
   server = gethostbyname(argv[1]);
   
   if (server == NULL) {
      fprintf(stderr,"No such host name - trying ip number...\n");
      server = gethostbyaddr((char *)inet_addr(argv[1]), 4, AF_INET);
      if(server == NULL) {
        perror("ERROR - No such host...\n");
        exit(0);
      }
   }
   
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   bcopy((char *)server->h_addr, (char *)&serv_addr.sin_addr.s_addr, server->h_length);
   serv_addr.sin_port = htons(portno);
   
   /* Now connect to the server */
   if (connect(sockfd, (struct sockaddr*)&serv_addr, sizeof(serv_addr)) < 0) {
      perror("ERROR connecting");
      sleep(10);
      exit(1);
   }
   sock = fdopen( sockfd, "r+");
   
   /* Read Version and Handle after Connect
   */
   fgets(buffer,512,sock);
   printf( "%s", buffer);
   if(buffer[0] == 'V' && buffer[strlen(buffer)-1] == '\n') {
     buffer[strlen(buffer)-1] == '\0';
     strncpy( version, buffer + 1, 20);
#ifdef DEBUG
     printf( "%s\n", version);
#endif
   } else {
     exit(1);
   }

   fgets(buffer,512,sock);
   printf( "%s", buffer);
   if(buffer[0] == 'H' && buffer[strlen(buffer)-1] == '\n') {
     buffer[strlen(buffer)-1] == '\0';
     strncpy( handle, buffer + 1, 20);
#ifdef DEBUG
     printf( "%s\n", handle);
#endif
   } else {
     exit(1);
   }
	
   /* Subscribe to
   */
   
   /* Send message to the server */
   n = write(sockfd, "C1|sub slice all\n", strlen("C1|sub slice all\n"));

   /* Send message to the server */
   n = write(sockfd, "C2|keepalive enable\n", strlen("C2|keepalive enable\n"));
   
   if (n < 0) {
      perror("ERROR writing to socket");
      exit(1);
   }
   
   /* Now read server responses */
   do {
     do {
       //Wait with timeout
       FD_ZERO( &rfds);
       FD_SET( sockfd, &rfds);

       /* Wait max 1 seconds. */
       tv.tv_sec = 1;
       tv.tv_usec = 0L;
       retval = select( sockfd + 1, &rfds, NULL, NULL, &tv);
     
       if( retval <= 0) { //Nothing new .. timeout?
         //Ping if timeout
         sprintf( buffer, "C%u|ping\n", sequence++);
         n = write(sockfd, buffer, strlen( buffer));
         lastping = time(NULL);
         if( n < 0) {
           fprintf( stderr, "ERROR ping\n");
           tellfrequency( -1.0);
           exit(1);    
         }
       } else {
         //Ping every second
         if( time(NULL) > (lastping + 1)) {
           sprintf( buffer, "C%u|ping\n", sequence++);
           n = write(sockfd, buffer, strlen( buffer));
           lastping = time(NULL);
           if( n < 0) {
             perror("ERROR ping\n");
             tellfrequency( -1.0);
             exit(1);
           }
         }
       }
     } while( retval <= 0);    

     bzero(buffer,2048);
     n = read(sockfd, buffer, 2048);
   
     if (n < 0) {
        perror("ERROR reading from socket");
        tellfrequency( -1.0);
        exit(1);
     }

#ifdef DEBUG	
     printf("%s\n",buffer);
     fflush(stdout);
#endif

     //Buffer could contain more than one line
     x = buffer;
     while( strlen(x) > 0) {  
       strncpy( line, x, 2048);
       *(char*)strchrnul( line, '\n') = '\0';
       x = (char*)strchrnul( x, '\n');
       if( *x) x++;
     
#ifdef DEBUG 
       printf( "-----\n%s\n---\n\n", line); 
#endif
       //Process TX (RX) Frequency of TX- or active Slices

       for( i = 0; i < 8; i++) {
         sprintf( comp, "|slice %d ", i);
         if( strstr( line, comp)) {
           if( strstr( line, "tx=1")) { slicetx = i; sliceactive = i; }
           if( strstr( line, "tx=0") && slicetx == i) slicetx = -1;
           if( strstr( line, "active=0") && sliceactive == i) sliceactive = -1;
           if( strstr( line, "in_use=1")) sliceactive = i;
	   if( strstr( line, "active=0")) slice_active[i] = 0;
	   if( strstr( line, "active=1")) { slice_active[i] = 1; sliceactive = i; }
           if( strstr( line, "tx=0")) slice_tx[i] = 0;
	   if( strstr( line, "tx=1")) slice_tx[i] = 1;

	   //TEST V3
           if( strstr( line, "in_use=0")) {
	     slice_tx[i] = 0;
	     slice_active[i] = 0;
             if( slicetx == i) slicetx = -1; 
	     if( sliceactive == i) sliceactive = -1;
	   }

           if( slicetx == -1 || sliceactive == -1) {
	     for( j = 0; j < 8; j++) {
	       if( slice_active[j]) sliceactive = j;
	       if( slice_tx[j]) slicetx = j;
	     }
	   }

           if( strstr( line, "RF_frequency=")) {
             sscanf( strstr( line, "RF_frequency=") + 13, "%lf", &slice_freq[i]);
	     //TEST V3
	     if( slice_tx[i] == 1) slicetx = i;  //More than one TX-enabled slice. Active last moved
	     if( slicetx == -1) {  //No TX-slice. Enable last moved slice
	       if( slice_tx[i] == 1) slicetx = i;
	       slice_active[i] = 1; 
	       sliceactive = i;
	     }
	   }
           if( strstr( line, "active=1")) sliceactive = i;
         }
       }
#ifdef DEBUG
       printf( "SLICE TX:%d SLICE ACTIVE:%d SLICE0=%lf SLICE1=%lf SLICE2=%lf SLICE3=%lf\n\n", slicetx, sliceactive, 
               slice_freq[0], slice_freq[1], slice_freq[2], slice_freq[3]);
       printf( "SLICE_TX: %d %d %d %d\n", slice_tx[0], slice_tx[1], slice_tx[2], slice_tx[3]);
       printf( "SLICE_ACTIVE: %d %d %d %d\n\n", slice_active[0], slice_active[1], slice_active[2], slice_active[3]);
#endif

       //Show used frequency to other processes 
       if( slicetx != -1) {
          tellfrequency( slice_freq[slicetx]);
       } else if( sliceactive != -1) {
         tellfrequency( slice_freq[sliceactive]);
       } else {
         tellfrequency( 0.0);
       }
     }

   } while( n > 0);
}
